@@ -12,10 +12,10 @@ File activity.cpp
 
 /***************************************************************************
  *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   This program is free software: you can redistribute it and/or modify  *
+ *   it under the terms of the GNU Affero General Public License as        *
+ *   published by the Free Software Foundation, either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  ***************************************************************************/
 
@@ -23,6 +23,8 @@ File activity.cpp
 #include "activity.h"
 #include "rules.h"
 
+#include <QSet>
+
 QString getActivityDetailedDescription(Rules& r, int id); //Implemented in timeconstraint.cpp
 
 GroupActivitiesInInitialOrderItem::GroupActivitiesInInitialOrderItem()
@@ -131,7 +133,7 @@ Activity::Activity()
 	comments=QString("");
 }
 
-Activity::Activity(
+/*Activity::Activity(
 	Rules& r,
 	int _id,
 	int _activityGroupId,
@@ -170,7 +172,7 @@ Activity::Activity(
 		assert(_nTotalStudents>=0);
 		this->nTotalStudents=_nTotalStudents;
 	}
-}
+}*/
 
 Activity::Activity(
 	Rules& r,
@@ -188,7 +190,7 @@ Activity::Activity(
 	int _computedNumberOfStudents)
 {
 	Q_UNUSED(r);
-	Q_UNUSED(_nTotalStudents);
+	//Q_UNUSED(_nTotalStudents);
 
 	comments=QString("");
 
@@ -203,7 +205,11 @@ Activity::Activity(
 	this->active=_active;
 	this->computeNTotalStudents=_computeNTotalStudents;
 	
-	assert(_computeNTotalStudents);
+	//assert(_computeNTotalStudents);
+	if(_computeNTotalStudents)
+		assert(_nTotalStudents==-1);
+	else
+		assert(_nTotalStudents==_computedNumberOfStudents);
 	this->nTotalStudents=_computedNumberOfStudents;
 }
 
@@ -310,6 +316,7 @@ void Activity::computeInternalStructure(Rules& r)
 	//teachers
 	//this->nTeachers=0;
 	this->iTeachersList.clear();
+	QSet<int> iTeachersSet;
 	for(QStringList::Iterator it=this->teachersNames.begin(); it!=this->teachersNames.end(); it++){
 		int tmp=r.teachersHash.value(*it, -1);
 		/*for(tmp=0; tmp<r.nInternalTeachers; tmp++){
@@ -319,7 +326,10 @@ void Activity::computeInternalStructure(Rules& r)
 		assert(tmp>=0 && tmp < r.nInternalTeachers);
 		//assert(this->nTeachers<MAX_TEACHERS_PER_ACTIVITY);
 		//this->teachers[this->nTeachers++]=tmp;
-		this->iTeachersList.append(tmp);
+		if(!iTeachersSet.contains(tmp)){
+			iTeachersSet.insert(tmp);
+			this->iTeachersList.append(tmp);
+		}
 	}
 
 	//subjects
@@ -339,6 +349,7 @@ void Activity::computeInternalStructure(Rules& r)
 	//students	
 	//this->nSubgroups=0;
 	this->iSubgroupsList.clear();
+	QSet<int> iSubgroupsSet;
 	for(QStringList::Iterator it=this->studentsNames.begin(); it!=this->studentsNames.end(); it++){
 		StudentsSet* ss=r.studentsHash.value(*it, NULL); //r.searchAugmentedStudentsSet(*it);
 		assert(ss);
@@ -353,7 +364,7 @@ void Activity::computeInternalStructure(Rules& r)
 			//assert(this->nSubgroups<MAX_SUBGROUPS_PER_ACTIVITY);
 			
 			bool duplicate=false;
-			if(this->iSubgroupsList.contains(tmp))
+			if(iSubgroupsSet.contains(tmp))
 			//for(int j=0; j<this->nSubgroups; j++)
 			//	if(this->subgroups[j]==tmp)
 					duplicate=true;
@@ -363,9 +374,11 @@ void Activity::computeInternalStructure(Rules& r)
 					.arg(this->id);
 				cout<<qPrintable(s)<<endl;*/
 			}
-			else
-				this->iSubgroupsList.append(tmp);
+			else{
 				//this->subgroups[this->nSubgroups++]=tmp;
+				iSubgroupsSet.insert(tmp);
+				this->iSubgroupsList.append(tmp);
+			}
 		}
 		else if(ss->type==STUDENTS_GROUP){
 			StudentsGroup* stg=(StudentsGroup*)ss;
@@ -381,7 +394,7 @@ void Activity::computeInternalStructure(Rules& r)
 				//assert(this->nSubgroups<MAX_SUBGROUPS_PER_ACTIVITY);
 
 				bool duplicate=false;
-				if(this->iSubgroupsList.contains(tmp))
+				if(iSubgroupsSet.contains(tmp))
 				//for(int j=0; j<this->nSubgroups; j++)
 				//	if(this->subgroups[j]==tmp)
 						duplicate=true;
@@ -391,9 +404,11 @@ void Activity::computeInternalStructure(Rules& r)
 						.arg(this->id);
 					cout<<qPrintable(s)<<endl;*/
 				}
-				else
+				else{
 					//this->subgroups[this->nSubgroups++]=tmp;
+					iSubgroupsSet.insert(tmp);
 					this->iSubgroupsList.append(tmp);
+				}
 			}
 		}
 		else if(ss->type==STUDENTS_YEAR){
@@ -412,7 +427,7 @@ void Activity::computeInternalStructure(Rules& r)
 					//assert(this->nSubgroups<MAX_SUBGROUPS_PER_ACTIVITY);
 
 					bool duplicate=false;
-					if(this->iSubgroupsList.contains(tmp))
+					if(iSubgroupsSet.contains(tmp))
 					//for(int j=0; j<this->nSubgroups; j++)
 					//	if(this->subgroups[j]==tmp)
 							duplicate=true;
@@ -424,6 +439,7 @@ void Activity::computeInternalStructure(Rules& r)
 					}
 					else{
 						//this->subgroups[this->nSubgroups++]=tmp;
+						iSubgroupsSet.insert(tmp);
 						this->iSubgroupsList.append(tmp);
 					}
 				}
