@@ -12,10 +12,10 @@ File fet.cpp - this is where the program FET starts
 
 /***************************************************************************
  *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   This program is free software: you can redistribute it and/or modify  *
+ *   it under the terms of the GNU Affero General Public License as        *
+ *   published by the Free Software Foundation, either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  ***************************************************************************/
 
@@ -41,6 +41,7 @@ File fet.cpp - this is where the program FET starts
 static QSet<QString> languagesSet;
 
 #include <ctime>
+#include <cstdlib>
 
 #include "timetableexport.h"
 #include "generate.h"
@@ -174,30 +175,32 @@ void usage(QTextStream* out, const QString& error)
 	s+=QString(
 		"Command line usage: \"fet-cl --inputfile=x [--outputdir=d] [--timelimitseconds=y] [--htmllevel=z] [--language=t] "
 		"[--writetimetableconflicts=wt1] "
-		"[--writetimetablesxml=wt2] "
-		"[--writetimetablesdayshorizontal=wt3] "
-		"[--writetimetablesdaysvertical=wt4] "
-		"[--writetimetablestimehorizontal=wt5] "
-		"[--writetimetablestimevertical=wt6] "
-		"[--writetimetablessubgroups=wt7] "
-		"[--writetimetablesgroups=wt8] "
-		"[--writetimetablesyears=wt9] "
-		"[--writetimetablesteachers=wt10] "
-		"[--writetimetablesteachersfreeperiods=wt11] "
-		"[--writetimetablesrooms=wt12] "
-		"[--writetimetablessubjects=wt13] "
-		"[--writetimetablesactivities=wt14] "
+		"[--writetimetablesstatistics=wt2] "
+		"[--writetimetablesxml=wt3] "
+		"[--writetimetablesdayshorizontal=wt4] "
+		"[--writetimetablesdaysvertical=wt5] "
+		"[--writetimetablestimehorizontal=wt6] "
+		"[--writetimetablestimevertical=wt7] "
+		"[--writetimetablessubgroups=wt8] "
+		"[--writetimetablesgroups=wt9] "
+		"[--writetimetablesyears=wt10] "
+		"[--writetimetablesteachers=wt11] "
+		"[--writetimetablesteachersfreeperiods=wt12] "
+		"[--writetimetablesrooms=wt13] "
+		"[--writetimetablessubjects=wt14] "
+		"[--writetimetablesactivities=wt15] "
 		"[--printactivitytags=a] [--printnotavailable=u] [--printbreak=b] [--dividetimeaxisbydays=v] [--duplicateverticalheaders=e] "
 		"[--printsimultaneousactivities=w] [--randomseedx=rx --randomseedy=ry] [--warnifusingnotperfectconstraints=s] "
-		"[--warnifusingstudentsminhoursdailywithallowemptydays=p] [--warnifusinggroupactivitiesininitialorder=g] [--verbose=r]\",\n"
+		"[--warnifusingstudentsminhoursdailywithallowemptydays=p] [--warnifusinggroupactivitiesininitialorder=g] [--warnsubgroupswiththesameactivities=ssa] [--verbose=r]\",\n"
 		"where:\nx is the input file, for instance \"data.fet\"\n"
 		"d is the path to results directory, without trailing slash or backslash (default is current working path). "
 		"Make sure you have write permissions there.\n"
 		"y is integer (seconds) (default 2000000000, which is practically infinite).\n"
 		"z is integer from 0 to 6 and represents the detail level for the generated HTML timetables "
 		"(default 2, larger values have more details/facilities and larger file sizes).\n"
-		"t is one of en_US, ar, ca, da, de, el, es, fa, fr, gl, he, hu, id, it, lt, mk, ms, nl, pl, pt_BR, ro, ru, si, sk, sq, sr, tr, uk, uz, vi, zh_CN (default en_US).\n"
-		"wt1 to wt14 are either true or false and represent whether you want the corresponding timetables to be written on the disk (default true).\n"
+		"t is one of en_US, ar, ca, cs, da, de, el, es, eu, fa, fr, gl, he, hu, id, it, lt, mk, ms, nl, pl, pt_BR, ro, ru, si, sk, sq, sr, tr, uk, "
+		"uz, vi, zh_CN, zh_TW (default en_US).\n"
+		"wt1 to wt15 are either true or false and represent whether you want the corresponding timetables to be written on the disk (default true).\n"
 		"a is either true or false and represets if you want activity tags to be present in the final HTML timetables (default true).\n"
 		"u is either true or false and represents if you want -x- (for true) or --- (for false) in the generated timetables for the "
 		"not available slots (default true).\n"
@@ -216,6 +219,8 @@ void usage(QTextStream* out, const QString& error)
 		"students min hours daily with allow empty days (default true).\n"
 		"g is either true or false, represents whether you want a message box to be shown, with a warning, if the input file contains nonstandard timetable "
 		"generation options to group activities in the initial order (default true).\n"
+		"ssa is either true or false, represents whether you want a message box to be show, with a warning, if your input file contains subgroups which have "
+		"the same activities (default true)."
 		"r is either true or false, represents whether you want additional generation messages and other messages to be shown on the command line (default false)."
 		"\n"
 		"Alternatively, you can run \"fet-cl --version [--outputdir=d]\" to get the current FET version. "
@@ -314,6 +319,7 @@ void readSimulationParameters()
 	
 	WRITE_TIMETABLE_CONFLICTS=newSettings.value("write-timetable-conflicts", "true").toBool();
 
+	WRITE_TIMETABLES_STATISTICS=newSettings.value("write-timetables-statistics", "true").toBool();
 	WRITE_TIMETABLES_XML=newSettings.value("write-timetables-xml", "true").toBool();
 	WRITE_TIMETABLES_DAYS_HORIZONTAL=newSettings.value("write-timetables-days-horizontal", "true").toBool();
 	WRITE_TIMETABLES_DAYS_VERTICAL=newSettings.value("write-timetables-days-vertical", "true").toBool();
@@ -339,6 +345,7 @@ void readSimulationParameters()
 	ENABLE_ACTIVITY_TAG_MAX_HOURS_DAILY=newSettings.value("enable-activity-tag-max-hours-daily", "false").toBool();
 	ENABLE_STUDENTS_MAX_GAPS_PER_DAY=newSettings.value("enable-students-max-gaps-per-day", "false").toBool();
 	SHOW_WARNING_FOR_NOT_PERFECT_CONSTRAINTS=newSettings.value("warn-if-using-not-perfect-constraints", "true").toBool();
+	SHOW_WARNING_FOR_SUBGROUPS_WITH_THE_SAME_ACTIVITIES=newSettings.value("warn-subgroups-with-the-same-activities", "true").toBool();
 	ENABLE_STUDENTS_MIN_HOURS_DAILY_WITH_ALLOW_EMPTY_DAYS=newSettings.value("enable-students-min-hours-daily-with-allow-empty-days", "false").toBool();
 	SHOW_WARNING_FOR_STUDENTS_MIN_HOURS_DAILY_WITH_ALLOW_EMPTY_DAYS=newSettings.value("warn-if-using-students-min-hours-daily-with-allow-empty-days", "true").toBool();
 	
@@ -351,6 +358,8 @@ void readSimulationParameters()
 	//MAIN_FORM_SHORTCUTS_TAB_POSITION=newSettings.value("FetMainForm/shortcuts-tab-position", "0").toInt();
 	MAIN_FORM_SHORTCUTS_TAB_POSITION=0; //always restoring to the first page, as suggested by a user
 	SHOW_SHORTCUTS_ON_MAIN_WINDOW=newSettings.value("FetMainForm/show-shortcuts", "true").toBool();
+
+	SHOW_TOOLTIPS_FOR_CONSTRAINTS_WITH_TABLES=newSettings.value("FetMainForm/show-tooltips-for-constraints-with-tables", "false").toBool();
 	
 	if(VERBOSE){
 		cout<<"Settings read"<<endl;
@@ -382,6 +391,7 @@ void writeSimulationParameters()
 	
 	settings.setValue("write-timetable-conflicts", WRITE_TIMETABLE_CONFLICTS);
 
+	settings.setValue("write-timetables-statistics", WRITE_TIMETABLES_STATISTICS);
 	settings.setValue("write-timetables-xml", WRITE_TIMETABLES_XML);
 	settings.setValue("write-timetables-days-horizontal", WRITE_TIMETABLES_DAYS_HORIZONTAL);
 	settings.setValue("write-timetables-days-vertical", WRITE_TIMETABLES_DAYS_VERTICAL);
@@ -407,6 +417,7 @@ void writeSimulationParameters()
 	settings.setValue("enable-activity-tag-max-hours-daily", ENABLE_ACTIVITY_TAG_MAX_HOURS_DAILY);
 	settings.setValue("enable-students-max-gaps-per-day", ENABLE_STUDENTS_MAX_GAPS_PER_DAY);
 	settings.setValue("warn-if-using-not-perfect-constraints", SHOW_WARNING_FOR_NOT_PERFECT_CONSTRAINTS);
+	settings.setValue("warn-subgroups-with-the-same-activities", SHOW_WARNING_FOR_SUBGROUPS_WITH_THE_SAME_ACTIVITIES);
 	settings.setValue("enable-students-min-hours-daily-with-allow-empty-days", ENABLE_STUDENTS_MIN_HOURS_DAILY_WITH_ALLOW_EMPTY_DAYS);
 	settings.setValue("warn-if-using-students-min-hours-daily-with-allow-empty-days", SHOW_WARNING_FOR_STUDENTS_MIN_HOURS_DAILY_WITH_ALLOW_EMPTY_DAYS);
 
@@ -418,6 +429,8 @@ void writeSimulationParameters()
 	//settings.setValue("FetMainForm/shortcuts-tab-position", MAIN_FORM_SHORTCUTS_TAB_POSITION);
 	//settings.setValue("FetMainForm/shortcuts-tab-position", 0); //always starting on the first page, as suggested by a user
 	settings.setValue("FetMainForm/show-shortcuts", SHOW_SHORTCUTS_ON_MAIN_WINDOW);
+
+	settings.setValue("FetMainForm/show-tooltips-for-constraints-with-tables", SHOW_TOOLTIPS_FOR_CONSTRAINTS_WITH_TABLES);
 }
 #endif
 
@@ -456,6 +469,9 @@ void initLanguagesSet()
 	languagesSet.insert("uz");
 	languagesSet.insert("sq");
 	languagesSet.insert("zh_CN");
+	languagesSet.insert("zh_TW");
+	languagesSet.insert("eu");
+	languagesSet.insert("cs");
 }
 
 #ifndef FET_COMMAND_LINE
@@ -639,6 +655,38 @@ void SomeQtTranslations()
 	Q_UNUSED(s7);
 	QString s8=QCoreApplication::translate("QDialogButtonBox", "N&o to All", "Accelerator key (letter after ampersand) for &OK, &Cancel, &Yes, Yes to &All, &No, N&o to All, must be different. Please keep the translation short.");
 	Q_UNUSED(s8);
+
+	QString s9=QCoreApplication::translate("QDialogButtonBox", "Help");
+	Q_UNUSED(s9);
+
+	//It seems that Qt 5 uses other context
+	QString s10=QCoreApplication::translate("QPlatformTheme", "&OK", "Accelerator key (letter after ampersand) for &OK, &Cancel, &Yes, Yes to &All, &No, N&o to All, must be different");
+	Q_UNUSED(s10);
+	QString s11=QCoreApplication::translate("QPlatformTheme", "OK");
+	Q_UNUSED(s11);
+	
+	QString s12=QCoreApplication::translate("QPlatformTheme", "&Cancel", "Accelerator key (letter after ampersand) for &OK, &Cancel, &Yes, Yes to &All, &No, N&o to All, must be different");
+	Q_UNUSED(s12);
+	QString s13=QCoreApplication::translate("QPlatformTheme", "Cancel");
+	Q_UNUSED(s13);
+	
+	QString s14=QCoreApplication::translate("QPlatformTheme", "&Yes", "Accelerator key (letter after ampersand) for &OK, &Cancel, &Yes, Yes to &All, &No, N&o to All, must be different");
+	Q_UNUSED(s14);
+	QString s15=QCoreApplication::translate("QPlatformTheme", "Yes to &All", "Accelerator key (letter after ampersand) for &OK, &Cancel, &Yes, Yes to &All, &No, N&o to All, must be different. Please keep the translation short.");
+	Q_UNUSED(s15);
+	QString s16=QCoreApplication::translate("QPlatformTheme", "&No", "Accelerator key (letter after ampersand) for &OK, &Cancel, &Yes, Yes to &All, &No, N&o to All, must be different");
+	Q_UNUSED(s16);
+	QString s17=QCoreApplication::translate("QPlatformTheme", "N&o to All", "Accelerator key (letter after ampersand) for &OK, &Cancel, &Yes, Yes to &All, &No, N&o to All, must be different. Please keep the translation short.");
+	Q_UNUSED(s17);
+
+	QString s18=QCoreApplication::translate("QPlatformTheme", "Help");
+	Q_UNUSED(s18);
+
+	QString s19=QCoreApplication::translate("QGnomeTheme", "&OK", "Accelerator key (letter after ampersand) for &OK, &Cancel, &Yes, Yes to &All, &No, N&o to All, must be different");
+	Q_UNUSED(s19);
+	QString s20=QCoreApplication::translate("QGnomeTheme", "&Cancel", "Accelerator key (letter after ampersand) for &OK, &Cancel, &Yes, Yes to &All, &No, N&o to All, must be different");
+	Q_UNUSED(s20);
+	
 }
 
 /**
@@ -760,6 +808,7 @@ int main(int argc, char **argv)
 		
 		WRITE_TIMETABLE_CONFLICTS=true;
 	
+		WRITE_TIMETABLES_STATISTICS=true;
 		WRITE_TIMETABLES_XML=true;
 		WRITE_TIMETABLES_DAYS_HORIZONTAL=true;
 		WRITE_TIMETABLES_DAYS_VERTICAL=true;
@@ -785,6 +834,8 @@ int main(int argc, char **argv)
 		
 		SHOW_WARNING_FOR_NOT_PERFECT_CONSTRAINTS=true;
 		
+		SHOW_WARNING_FOR_SUBGROUPS_WITH_THE_SAME_ACTIVITIES=true;
+		
 		SHOW_WARNING_FOR_STUDENTS_MIN_HOURS_DAILY_WITH_ALLOW_EMPTY_DAYS=true;
 		
 		SHOW_WARNING_FOR_GROUP_ACTIVITIES_IN_INITIAL_ORDER=true;
@@ -843,6 +894,10 @@ int main(int argc, char **argv)
 				if(s.right(5)=="false")
 					SHOW_WARNING_FOR_NOT_PERFECT_CONSTRAINTS=false;
 			}
+			else if(s.left(37)=="--warnsubgroupswiththesameactivities="){
+				if(s.right(5)=="false")
+					SHOW_WARNING_FOR_SUBGROUPS_WITH_THE_SAME_ACTIVITIES=false;
+			}
 			else if(s.left(53)=="--warnifusingstudentsminhoursdailywithallowemptydays="){
 				if(s.right(5)=="false")
 					SHOW_WARNING_FOR_STUDENTS_MIN_HOURS_DAILY_WITH_ALLOW_EMPTY_DAYS=false;
@@ -860,6 +915,10 @@ int main(int argc, char **argv)
 					WRITE_TIMETABLE_CONFLICTS=false;
 			}
 			//
+			else if(s.left(28)=="--writetimetablesstatistics="){
+				if(s.right(5)=="false")
+					WRITE_TIMETABLES_STATISTICS=false;
+			}
 			else if(s.left(21)=="--writetimetablesxml="){
 				if(s.right(5)=="false")
 					WRITE_TIMETABLES_XML=false;
@@ -967,13 +1026,13 @@ int main(int argc, char **argv)
 	
 			//QString qv=qVersion();
 			out<<"FET version "<<qPrintable(FET_VERSION)<<endl;
-			out<<"Free timetabling software, licensed under GNU GPL v2 or later"<<endl;
-			out<<"Copyright (C) 2002-2014 Liviu Lalescu, Volker Dirr"<<endl;
+			out<<"Free timetabling software, licensed under the GNU Affero General Public License version 3 or later"<<endl;
+			out<<"Copyright (C) 2002-2015 Liviu Lalescu, Volker Dirr"<<endl;
 			out<<"Homepage: http://lalescu.ro/liviu/fet/"<<endl;
 			//out<<" (Using Qt version "<<qPrintable(qv)<<")"<<endl;
 			cout<<"FET version "<<qPrintable(FET_VERSION)<<endl;
-			cout<<"Free timetabling software, licensed under GNU GPL v2 or later"<<endl;
-			cout<<"Copyright (C) 2002-2014 Liviu Lalescu, Volker Dirr"<<endl;
+			cout<<"Free timetabling software, licensed under the GNU Affero General Public License version 3 or later"<<endl;
+			cout<<"Copyright (C) 2002-2015 Liviu Lalescu, Volker Dirr"<<endl;
 			cout<<"Homepage: http://lalescu.ro/liviu/fet/"<<endl;
 			//cout<<" (Using Qt version "<<qPrintable(qv)<<")"<<endl;
 
