@@ -8,10 +8,10 @@
 
 /***************************************************************************
  *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   This program is free software: you can redistribute it and/or modify  *
+ *   it under the terms of the GNU Affero General Public License as        *
+ *   published by the Free Software Foundation, either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  ***************************************************************************/
 
@@ -121,9 +121,9 @@ void HelpFaqForm::setText()
 	s+="\n\n";
 
 	s+=tr("Q: Is it possible to use non-integer weights for constraints?\n\n"
-		"A: Yes. Using values like 99.75% might be good sometimes. "
-		"The precision in FET is limited to %1 decimal digits after the decimal point, "
-		"but probably nobody will use such a fine precision").arg(CUSTOM_DOUBLE_PRECISION);
+		"A: Yes. If a constraint is allowed values under 100%, you can use any weight, even fractional numbers like 99.75%. It might help "
+		"in constraints like min days, preferred rooms or max hours daily. The precision in FET is limited to %1 decimal digits after the decimal point, "
+		"but probably nobody will use such a fine precision.").arg(CUSTOM_DOUBLE_PRECISION);
 
 	s+="\n\n";
 	s+="--------------------";
@@ -320,15 +320,6 @@ void HelpFaqForm::setText()
 	s+="--------------------";
 	s+="\n\n";
 
-	s+=tr("Q: Is it allowed to use non-integer weights?\n\n"
-		"A: If a constraint is allowed values under 100%, you can use any weight, even fractional numbers like 99.75%. It might help "
-		"in constraints like min days, preferred rooms or max hours daily.");
-
-	s+="\n\n";
-	s+="--------------------";
-	s+="\n\n";
-
-
 	s+=tr("Q: I have a sports room which allows more than 1 activity simultaneously in it. How to make the preferred room constraints? "
 		"Can FET accept more than one activity at the same time in a single room?\n\n"
 		"A: Each room can host a single activity at the same time. How to implement what you need? You can add more "
@@ -350,14 +341,36 @@ void HelpFaqForm::setText()
 	s+=tr("Q: Is it possible to work with 0 hour?\n\n"
 		"A: A bit difficult. You have to choose a day for this 0 hour. Then add breaks or not available to "
 		"prevent other activities in other days at hour 0, then add students set not available to prevent other "
-		"students set from having hours at this hour 0. Or variants of this.\n\n"
-		"Mr. Zsolt Udvari used another trick: considered the last hour to be hour 0. But this is not always applicable.");
+		"students sets from having hours at this hour 0. Or variants of this.\n\n"
+		"Zsolt Udvari used another trick: considered the last hour to be hour 0. But this is not always applicable.");
 
 	s+="\n\n";
 	s+="--------------------";
 	s+="\n\n";
 
-	s+=tr("Q: How does FET care about weights. What do they mean?\n\n"
+	s+=tr("Q: How does FET care about weights? What do they mean?\n\n"
+		"A: The weights under 100% are subjective, because FET can skip them if "
+		"necessary.\n\n"
+		"Volker Dirr tried to express how FET considers under 100% weights:\n\n"
+		"weight = 50% means: In average FET retries two times to place an "
+		"activity without a conflict. If it isn't able to place the activity "
+		"without a conflict after average 2 times it keeps the conflict and tries "
+		"to place the next activity.\n"
+		"weight = 75% means: In average FET retries four times to place an "
+		"activity without a conflict. If it isn't able to place the activity "
+		"without a conflict after average 4 times it keeps the conflict and tries "
+		"to place the next activity.\n"
+		"weight = 99% means: In average FET retries 100 times to place an "
+		"activity without a conflict. If it isn't able to place the activity "
+		"without a conflict after average 100 times it keeps the conflict and "
+		"tries to place the next activity.\n"
+		"weight = 99.99% means: In average FET retries 10000 times to place an "
+		"activity without a conflict. If it isn't able to place the activity "
+		"without a conflict after average 10000 times it keeps the conflict and "
+		"tries to place the next activity.\n\n"
+		"Also, activities might get unallocated, and the cycle would be opened.");
+
+	/*s+=tr("Q: How does FET care about weights. What do they mean?\n\n"
 		"A: The weights under 100% are subjective, because FET can skip them if necessary.\n\n"
 		"Volker Dirr tried to express how FET considers under 100% weights, but his words are not completely reflecting the fact:\n\n"
 		"Here is a comment from Volker Dirr:\n\n"
@@ -369,7 +382,7 @@ void HelpFaqForm::setText()
 		"If it isn't able to place the activity without a conflict after average 100 times it keeps the conflict and tries to place the next activity.\n"
 		"weight = 99.99% means: In average FET retries 10000 times to place an activity without a conflict."
 		" If it isn't able to place the activity without a conflict after average 10000 times it keeps the conflict and tries to place the next activity.\n\n"
-		"This is not 100% correct. Activities might get unallocated, and cycle reopened.");
+		"This is not 100% correct. Activities might get unallocated, and cycle reopened.");*/
 
 	s+="\n\n";
 	s+="--------------------";
@@ -487,10 +500,10 @@ void HelpFaqForm::setText()
 		"activities equal to the number of days per week. Example: 7 activities with duration 1 in a 5 days week, then "
 		"transform into 5 activities with durations: 2,2,1,1,1 and add a single container activity with these 5 "
 		"components (possibly raising the weight of added constraint min days between activities up to 100%)\n\n"
-		"2. If you don't add 'force consecutive if same day', then add a larger activity splitted into a number of "
-		"activities equal with the number of days per week and the remaining components into other larger splitted activity. For "
+		"2. If you don't add 'force consecutive if same day', then add a larger activity split into a number of "
+		"activities equal with the number of days per week and the remaining components into other larger split activity. For "
 		"example, suppose you need to add 7 activities with duration 1 in a 5 days week. Add 2 larger container activities, "
-		"first one splitted into 5 activities with duration 1 and second one splitted into 2 activities with duration 1 ("
+		"first one split into 5 activities with duration 1 and second one split into 2 activities with duration 1 ("
 		"possibly raising the weight of added constraints min days between activities for each of the 2 containers up to 100%)");
 
 	s+="\n\n";
@@ -612,6 +625,9 @@ void HelpFaqForm::setText()
 		"Hour 11:00 Break\n"
 		"Hour 12:00 Math (second hour of act. 100)?\n\n"
 		"A: No, the activity must respect the break, so it is before or after the break with all the hours of it.");
+	s+=" ";
+	s+=tr("Alternative solutions: either you can split that activity into two subactivities with duration 1 (without a min days constraint "
+		"between them), and add a two activities consecutive constraint, or you can choose to not add the break constraint in your file.");
 
 	s+="\n\n";
 	s+="--------------------";
@@ -904,7 +920,7 @@ void HelpFaqForm::setText()
 	s+=tr("A: You can activate/deactivate a constraint by clicking the 'Activate'/'Deactivate' buttons in the all time/space"
 	 " constraints dialogs.");
 
-	s+="\n\n";
+	/*s+="\n\n";
 	s+="--------------------";
 	s+="\n\n";
 
@@ -912,7 +928,7 @@ void HelpFaqForm::setText()
 	s+="\n\n";
 	s+=tr("A: You can sort the constraints in the all time/space constraints dialogs. Sorting is done ascending, according to each"
 	 " constraints' comments. You can add suitable comments to obtain a desired order, like: 'rank #1 ... other comments' and"
-	 " 'rank #2 ... other different comments'.");
+	 " 'rank #2 ... other different comments'.");*/
 
 	s+="\n\n";
 	s+="--------------------";
@@ -946,12 +962,12 @@ void HelpFaqForm::setText()
 	s+="--------------------";
 	s+="\n\n";
 
-	s+=tr("Q: I have many subgroups and I don't explicitely use them. Can I hide them in the combo boxes, to add easier the constraints?"
+	s+=tr("Q: I have many subgroups and I don't explicitly use them. Can I hide them in the combo boxes, to add easier the constraints?"
 	 " Also, can I make the FET interface work faster?");
 	s+="\n\n";
 	s+=tr("A:", "Answer");
 	s+=" ";
-	s+=tr("If you have many subgroups and you don't explicitely use them, it is recommended to use the three global settings: hide subgroups"
+	s+=tr("If you have many subgroups and you don't explicitly use them, it is recommended to use the three global settings: hide subgroups"
 		" in combo boxes, hide subgroups in activity planning, and do not write subgroups timetables on hard disk.");
 	s+="\n";
 	s+=tr("Note that using the global menu setting to hide subgroups in activity planning is a different thing from the check box in the activity"
@@ -988,6 +1004,11 @@ void HelpFaqForm::setText()
 	s+=tr("Maybe an alternative to dividing a year into many categories/subgroups would be to enter individual students as FET subgroups and add into"
 		" each group the corresponding subgroups. But this is hard to do from the FET interface - maybe a solution would be to use an automatic"
 		" tool to convert your instition data into a file in .fet format.");
+	s+=" ";
+	s+=tr("Or you might use the FET feature to import students sets from comma separated values (CSV) files.");
+	s+=" ";
+	s+=tr("In such cases (individual students as FET subgroups), remember that a smaller number of total subgroups means faster generation time, so"
+		" you might want to consider a single subgroup for two or more students who have the exact same activities and constraints.");
 
 	plainTextEdit->setPlainText(s);
 }
